#include <queue>
#include <stack>
#include <fstream>
#include <iostream>

#include "BvhLoader.h"



namespace CHCDemoEngine
{

using namespace std;


#define TYPE_INTERIOR -2
#define TYPE_LEAF -3



BvhNode *BvhLoader::LoadNextNode(ifstream &stream, BvhInterior *parent)
{
	int nodeType;
	stream.read(reinterpret_cast<char *>(&nodeType), sizeof(int));

	BvhNode *node;

	if (nodeType == TYPE_LEAF)
		node = new BvhLeaf(parent);
	else if (nodeType == TYPE_INTERIOR)
		node = new BvhInterior(parent);
	else
		cerr << "error: wrong node type: " << nodeType << endl;
		

	Vector3 bMin, bMax;

	stream.read(reinterpret_cast<char *>(&(node->mFirst)), sizeof(int));
	stream.read(reinterpret_cast<char *>(&(node->mLast)), sizeof(int));

	stream.read(reinterpret_cast<char *>(&bMin), sizeof(Vector3));
	stream.read(reinterpret_cast<char *>(&bMax), sizeof(Vector3));

	node->mBox = AxisAlignedBox3(bMin, bMax);
	node->mArea = node->mBox.SurfaceArea();

	//cout << "box: " << node->mBox << " area: " << node->mArea << endl;
	
	return node;
}


Bvh *BvhLoader::Load(const string &filename,  
					 const SceneEntityContainer &entities)
{
	queue<BvhNode *> tQueue;
	ifstream stream(filename.c_str(), ifstream::binary);

	if (!stream.is_open()) return NULL;

	cout << "loading bvh" << endl;

	Bvh *bvh = new Bvh(entities);
	bvh->mRoot = LoadNextNode(stream, NULL);

	tQueue.push(bvh->mRoot);
	bvh->mNumNodes = 1;

	while(!tQueue.empty())
	{
		BvhNode *node = tQueue.front();
		tQueue.pop();

		if (!node->IsLeaf())
		{
			bvh->mNumNodes += 2;

			BvhInterior *interior = static_cast<BvhInterior *>(node);

			BvhNode *front = LoadNextNode(stream, interior);
			BvhNode *back = LoadNextNode(stream, interior);

			interior->mFront = front;
			interior->mBack = back;

			front->mDepth = interior->mDepth + 1;
			back->mDepth = interior->mDepth + 1;

			tQueue.push(front);			
			tQueue.push(back);
		}
	}

	cout << "... finished loading " << bvh->mNumNodes << " nodes, updating boxes" << endl;

	///////////
	//-- post process nodes

	bvh->PostProcess();
	// set virtual leaves for 500 triangles
	bvh->SetVirtualLeaves(0);

	bvh->UpdateNumLeaves(bvh->mRoot);
	// compute unique ids
	bvh->ComputeIds();
	// specify bounds for occlusion tests
	bvh->RecomputeBounds();
	// compute and print stats
	bvh->ComputeBvhStats();
	bvh->PrintBvhStats();

	return bvh;
}


}
