#ifndef __MATERIAL_H
#define __MATERIAL_H


namespace CHCDemoEngine 
{

class Texture;
class RenderState;


class RgbaColor 
{

public:

	float r, g, b, a;

	RgbaColor(): r(1), g(1), b(1), a(1)
	{}

	RgbaColor(float _r, float _g, float _b, float _a): r(_r), g(_g), b(_b), a(_a)
	{}

	friend RgbaColor RandomColor(float a = 0.0f, float b = 1.0f);
};


// Forward declarations
RgbaColor RandomColor(float a, float b);



class Material 
{
public:

	Material();

	Material(int id);

	Material(const RgbaColor &color);
	/** Returns unique material id.
	*/
	int GetId() const;

	friend Material RandomMaterial();

	inline Texture *GetTexture() const { return mTexture; }

	void SetTexture(Texture *texture) { mTexture = texture; }
	/** Renders this material.
	*/
	void Render(RenderState *state);


	///////////

	RgbaColor mDiffuseColor;
	RgbaColor mSpecularColor;
	RgbaColor mAmbientColor;

	bool mAlphaTestEnabled;

protected:

	/** Initialize the material with default values
	*/
	void InitMaterial();

	////////////////

	/// unique material id
	int mId;
	/// the assciated texture
	Texture *mTexture;
};


extern Material RandomMaterial();

}

#endif
