#include "RenderQueue.h"
#include "SceneEntity.h"
#include "Geometry.h"
#include "Texture.h"
#include "Material.h"


using namespace std;


namespace CHCDemoEngine
{


inline static bool IsLower(SceneEntity *ent1, SceneEntity *ent2)
{
	// group by texture size
	Texture *t1 = ent1->GetMaterial()->GetTexture();
	Texture *t2 = ent2->GetMaterial()->GetTexture();

	int tsize1 = t1 ? t1->GetByteSize() : 0;
	int tsize2 = t2 ? t2->GetByteSize() : 0;

	// if texture size equal, group by material
	if (tsize1 == tsize2)
		return ent1->GetMaterial() < ent2->GetMaterial();

	return tsize1 < tsize2;
}


RenderQueue::RenderQueue(): mState(NULL), mMinSizeForSorting(3)
{
}


RenderQueue::RenderQueue(RenderState *state):
mState(state)
{
}


void RenderQueue::Enqueue(SceneEntity *entity)
{
	mEntities.push_back(entity);
}
	

void RenderQueue::Clear()
{
	mEntities.clear();

}


void RenderQueue::SetRenderState(RenderState *state)
{
	mState = state;
}


void RenderQueue::Render()
{
	if (mEntities.size() >= mMinSizeForSorting)
		Sort();
	
	SceneEntityContainer::const_iterator sit, sit_end = mEntities.end();

	for (sit = mEntities.begin(); sit != sit_end; ++ sit)
	{
		SceneEntity *ent = *sit;
		ent->Render(mState);
	}
}
	

void RenderQueue::Sort()
{
	sort(mEntities.begin(), mEntities.end(), IsLower);
}


}