#ifndef NO_PRAGMA_ONCE
#pragma once
#endif

#ifndef __TEXTURE_H
#define __TEXTURE_H

#include <string>

namespace CHCDemoEngine
{

///
class Texture
{
public:
	/** Constructor loading texture from file
	*/
	Texture(const std::string &filename);
	/** Destroys data.
	*/
	~Texture();
	/** Returns image data.
	*/
	void *GetImage() const;
	
	enum { FORMAT_INVALID, FORMAT_RGB, FORMAT_RGBA };
	
	/** Returns width of texture.
	*/
	int GetWidth() const;
	/** Returns width of texture.
	*/
	int GetHeight() const;
	/** Returns texture format. Currently only RGBA supported.
	*/
	int GetFormat() const;
	/** Returns name of the texture
	*/
	std::string GetName() const;

	int GetRowLength() const;

	int GetPixelSize() const;
	
	inline int GetByteSize() const;
	/** Bind the texture.
	*/
	void Bind() const;



protected:

	void Create();

	int mFormat;
	int mWidth;
	int mHeight;

	unsigned int mTexId;

	std::string mName;

	int mByteSize;
	void *mImage;
};


inline int Texture::GetByteSize() const
{
	return mByteSize;
}

}

#endif // __TEXTURE_H
