#ifndef __VISUALIZATION_H
#define __VISUALIZATION_H

#include "common.h"


namespace CHCDemoEngine
{

class Bvh;
class Camera;
class RenderState;
class AxisAlignedBox3;


/** Abstract class implementing a scene traversal for rendering.
*/
class Visualization
{
public:
	
	Visualization(Bvh *bvh, Camera *camera, Camera *vizCamera, RenderState *renderState);

	~Visualization() {}

	/** Renders the visualizatioin
	*/
	void Render();
	/** Sets the scene hierarchy.
	*/
	void SetHierarchy(Bvh *bvh);
	/** Sets the visualization camera.
	*/
	void SetVizCamera(Camera *cam)  {mVizCamera = cam;}
	/** Sets the currently used scene camera.
	*/
	void SetCamera(Camera *cam)  {mCamera = cam;}
	/** Sets the current render state
	*/
	void SetRenderState(RenderState *state);
	

protected:

	/** Renders a camera frustum.
	*/
	void RenderFrustum();
	/** Renders a bounding box for visualization.
	*/
	void RenderBox(const AxisAlignedBox3 &box);



	////////////
	//-- members

	/// the current camera
	Camera *mCamera;
	/// the visualization camera
	Camera *mVizCamera;
	/// the root of the scene hierarchy
	Bvh *mBvh;
	/// the current render state
	RenderState *mRenderState;
};

}



#endif // VISUALIZATION_H